\ For noForth C&V2553 lp.0, bit input and output, timer interrupt and hardware
\ interrupt with machine code, on MSP430G2553 using port-1 and port-2.

\ RC5 remote control decoder and power control for MSP430 Launchpad.
\ RC-LAMP puts the received 6-bits key-code on leds.
\ Example of two cooperating interrupt routines.

( Connect an 36 KHz infrared receiver to the RC5 connector,
  it must be a 3 Volt version like TSOP34536. As output a power
  led at the LED connector with a 47 Ohm series resistor. Or at 
  the PWR connector max. 2 Amp. At P1.2 to P2.7 the led-print must 
  be connected. )

( The settings for P2.0 will be found from page 42 onward of SLAS735E.PDF,
  settings for timer A0 can be found on page 378 onward of SLAU144I.PDF )

\ P2.0 is used as RC5 input.
\ 01 28 - P2 RC5 receiver input           \ P2.0
\ FE 29 - P2 output bits
\ FE 2A - P2 direction register
\ 01 2B - P2 Interrupt flag
\ 01 2C - P2 Interrupt edge
\ 01 2D - P2 Interrupt enable
\ 01 2E - P2 configuration bits 0
\ 01 2F - P2 resistance on/off
\ 01 42 - P2 configuration bits 1
\ FFE6  - P2 Interrupt vector

\ Adresses of Timer-A0
\ 160 = TA0CTL   - Timer A0 control
\ 162 = TA0CCTL0 - Timer A0 Comp/Capt. control 0
\ 172 = TA0CCR0  - Timer A0 Comp/Capt. 0
\ 170 = TA0R     - Timer A0 register
\ 0000 0010 1101 0100 = 02D4 - TA = zero, count up mode, SMCLK, presc. /8
\ FFF2   - Timer A0 Interrupt vector

\ LED1  - P1.0 Red
\ S2    - P1.3 Switch
\ LED2  - P1.6 Green
\ Trace - P2.2 Testpoint
\ Uit   - P2.0 RC5 input
\ LED2  - P2.1 Uitput

HEX

\ Special output variant which saves bit 0 and 1 of P2
\ Uses bit P2.2 tot P2.7 for 6 leds!!
: >LEDS     FC 29 *bic  2* 2* 29 *bis ;  ( b -- )
: RED       01 21 ;                    \ Red led at P1.0
: GREEN     40 21 ;                    \ Green led at P1.6
: LAMP      02 29 ;                    \ Solid state relais at P2.1
: POWER     10 29 ;                    \ Power output at P2.4 (2 Amp. max.)
: FLASH     -1 >leds 200 ms  0 >leds 200 ms ;

\ Bit 6 t/m 10 are the system-bits they contain the device address
\ TV = 00 -> 0000, CD = 20 -> 0500 en VCR = 05 -> 0140
dm 00  6 lshift  constant RCTYPE        \ RC5 transmitter type
value RCDATA                            \ Command databyte (14 bits)
value RC#                               \ Number of received bits

\ : ADR               ( ccc -- Ram-addr )     ' >body @ ;
\ Give true when there is a valid RC5 command is for this device.
value RCKEY?        ( -- flag )

code INTERRUPT-ON   D232 , 4F00 ,  end-code
code INTERRUPT-OFF   C232 ,  4F00 , end-code

code RCBIT-IN   40B2 ,  370 ,
172 ,  5392 ,  adr rc# ,  B392 ,
adr rc# ,  2C24 ,  1208 ,  4218 ,
adr rcdata ,  B3D2 ,  28 ,  6308 ,
B318 ,  2814 ,  90B2 ,  1C ,
adr rc# ,  200F ,  4218 ,  adr rcdata ,
F038 ,  7C0 ,  9038 ,  rctype ,
2008 ,  43B2 ,  adr rckey? ,  C3D2 ,
2B ,  D3D2 ,  2D ,  4382 ,
160 ,  3C06 ,  C3D2 ,  2B ,
D3D2 ,  2D ,  4382 ,  160 ,
4138 ,  3C05 ,  B3D2 ,  28 ,
6292 ,  adr rcdata ,  adr rcdata ,  1300 ,
end-code

code RCSTART   9382 ,  adr rckey? ,
200C ,  C3D2 ,  2D ,  40B2 ,
1AE ,  172 ,  40B2 ,  2D4 ,
160 ,  4392 ,  adr rc# ,  4392 ,
adr rcdata ,  C3D2 ,  2B ,  1300 ,
end-code

: RC-ON             ( -- )          \ Install decoder hardware
    0000 160 !  0010 162 !          \ Timer A0 off and compare 0 interrupt on
\ Set hardware interrupt at P2.0 ready, all other bits of P2 are outputs
    00 02E c!                       \ Port-2 use all bits as normal I/O
    01 2F *bis                      \ Bit-0 resistor on
    FE 2A c!                        \ Bit-0 is input, 1 t/m 7 are output
    01 29 *bis                      \ Bit-0 pullup resistance
    01 2C *bis                      \ Bit-0 falling edge
    01 2B *bic                      \ Bit-0 reset HW interrupt flag
    01 2D *bis                      \ Bit-0 interrupt on
    0 to rckey?                     \ Allow new key input
    interrupt-on                    \ Activate decoder
    ;

: RC-OFF            ( -- )
    interrupt-off                   \ Deactivate decoder
    0000 160 !  0000 162 !          \ Stop timer-A0
    01 2D *bic                      \ HW interrupts off
    ;

\ Receive a RC5 databyte, x is a code from the definined RC5-code
\ command set. More info search for RC5 device documentation
\ here: https://en.wikipedia.org/wiki/RC-5
: RCKEY             ( -- x )        \ Wait for an RC5 command
    begin  rckey? until             \ Wait for valid key
    rcdata 3F and                   \ Low 6-bits are key-code
    0 to rckey?                     \ Allow next key
    ;

: RC-LAMP           ( -- )          \ Control a lamp thru rc5
    rc-on  flash                    \ Show startup 
    lamp *bic  green *bic           \ (Led)lamp and green led off
    begin
        rckey 0C = if               \ On/Off key received ?
            0E0 ms  rckey? if       \ Yes, wait a moment, check for key
                rckey drop          \ Throw repeated key away!
            then
            lamp *bix               \ Switch lamp On/Off
        else
            green *bix              \ No, switch GREEN-led On/Off
        then
    key? until  rc-off
    lamp *bic  green *bic           \ Outputs off
    ;

: RC-POWER          ( -- )          \ Control power ouput thru RC5
    rc-on  flash                    \ Show startup 
    power *bic  green *bic          \ (Led)lamp and green led off
    begin
        rckey 0C = if               \ On/Off key received ?
            0E0 ms  rckey? if       \ Yes, wait a moment, check for key
                rckey drop          \ throw repeated code away!
            then
            power *bix              \ Switch power output On/Off
        else
            green *bix              \ No, switch GREEN-led On/Off
        then
    key? until  rc-off              \ Ready
    power *bic  green *bic          \ Outputs off
    ;

: RC-TEST           ( -- )          \ Show received RC5 keycodes
    rc-on  flash  hex               \ Show startup
    begin
        rckey  dup >leds  cr        \ Get rc-key, show on leds and
        dup 2 .r space              \ show hex and
        decimal 4 .r space  hex     \ decimal on screen
    key? until  rc-off  0 >leds     \ Ready, leds off
    ;

' rcbit-in >body  FFF2 vec!         \ Set interrupt vectors
' rcstart >body   FFE6 vec!

' rc-lamp  to app  freeze

\ End
