\ For noForth C&V2553 lp.0, bitbang SPI on MSP430G2553 using port-2.
\ Control of seven segment display's using one or more 74HC4094 chips.

( Connect a 74HC4094 to VCC and ground, pin-1 to P2.0, pin-2 to P2.1
  pin-3 to P2.2!! Pin-4 to pin-11 are eight outputs they must be connected
  to an 7-segm. display. For more digits multiple 74HC4094 maybe 
  connected to each other, strobe and cp in parallel. See schematic. )

hex
\ 029 - P2OUT is the port 2 output register
\ 02A - P2DIR is the port 2 direction register
: READY     01 029 ;            \ Bit P2.0 - strobe 74HC4094
: DATA      02 029 ;            \ Bit P2.1 - data 74HC4094
: CLOCK     04 029 ;            \ Bit P2.2 - cp 74HC4094
: INIT-SPI  07 02A *bis ;

( u*8 bits output uses no more than only 3 I/O-bits )
: WRITE-BIT       ( flag -- )         ( Write one bit to 74HC4094 )
    if  data *bis else data *bic  then  clock *bic clock *bis ;

: SHOW              ( -- )              ( Set data on 74HC4094 outputs )
    ready *bis ready *bic ;
 
: >DRIVER           ( x -- )            ( Schrijf x naar display )
    8 0 do                              ( display driver )
        dup 80 and write-bit  2*
    loop
    drop ;

( Make numbers for an seven segment display using 74HC4094 0 t/m F and dot )
( Layout of 7-segm. Display  +-04-+ )
(                            20  08 )
( Driver 74HC4094            +-10-+ )
(                            40  01 )
( Dec. dot is 02 !!          +-80-+ )
CREATE NUMBERS         ( Building characters )
  bn 00010010 c, bn 11110110 c, bn 00100011 c, bn 01100010 c, ( 0, 1, 2, 3 )
  bn 11000110 c, bn 01001010 c, bn 00001010 c, bn 11110010 c, ( 4, 5, 6, 7 )
  bn 00000010 c, bn 01000010 c, bn 10000010 c, bn 00001110 c, ( 8, 9, A, b )
  bn 00011011 c, bn 00100110 c, bn 00001011 c, bn 10001011 C, ( C, d, E, F )
  bn 11101111 c, bn 11111101 c, bn 11111111 c,  align         ( -, ., and off )

: >DISPLAY          ( u -- )            ( Send number u to digits )
    12 umin  numbers   +                ( From binary to 7-segment )
    c@  >driver  show ;                 ( And show it ! )

: FLASH         ( -- )                  ( Show startup: F and - )
    0F >display  150 ms
    10 >display  150 ms
    12 >display  150 ms ;

: CHARACTERS    ( -- )                  ( Show all implemented characters )
    init-spi  flash                     ( Initialise 7-segm. example )
    0                                   ( Character counter on stack )
    begin
        dup >display  1+                ( Show character and incr. counter )
        150 ms                          ( Wait a moment )
        dup 13 = if  drop  0  then      ( Count 18? start at zero )
    key? until  drop                    ( Stop or loop again )
    12 >display                         ( Display off )
    ;

: COUNTER       ( -- )                  ( Decimal counter )
    init-spi  flash                     ( Initialise 7-segm. example )
    begin
        0A 0 do i >display 200 ms loop  ( Count )
    key? until
    12 >display ;                       ( Display off )

: DICE          ( -- )                  ( Throw a single dice )
    init-spi  10 >display               ( Initialise 7-segm. example )
    begin
        0  begin  2 ms  1+  key? until  ( Wait for key, increase counter ) 
        key drop                        ( Remove key )
        7 and  5 min 1+ >display 20 ms  ( Show only numbers from 1 to 6 )
    key? until  12 >display ;           ( Display off )
         
freeze

( End )
